/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Filters
{
    public interface IFilter
    {
        /// <summary>
        /// Szuro kiszamoltatasa a kepre
        /// </summary>
        /// <param name="bitmap">Ezen a Bitmap-en hajtodik vegre a szuro</param>
        /// <param name="progressEvent">a folyamat kijelzesehez</param>
        /// <param name="finishedEvent">a muvelet befejezesekor</param>
        void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent);
    }

    public delegate void ProgressEventHandler(object sender, ProgressEventArgs e);
    public delegate void FinishedEventHandler(object sender, EventArgs e);

    public class ProgressEventArgs : EventArgs
    {
        private const string INPROGRESS = "Szr kiszmolsa folyamatban...";
        float progress;
        string message;

        public ProgressEventArgs(float newProgress, string newMessage)
        {
            progress = newProgress;
            message = newMessage;
        }

        public ProgressEventArgs(float newProgress) : this(newProgress, INPROGRESS) { }

        public float Progress
        {
            get { return progress; }
        }

        public string Message
        {
            get { return message; }
        }

        public static ProgressEventArgs InProgress
        {
            get { return new ProgressEventArgs(0, INPROGRESS); }
        }
	}

    public class BitmapOperations
    {
        public static void CopyBitmapToBitmap(Bitmap src, Bitmap dest)
        {
            Graphics g = Graphics.FromImage(dest);
            g.DrawImageUnscaledAndClipped(src, new Rectangle(0, 0, src.Width, src.Height));
            g.Dispose();
        }

        public static byte ClampIntegerToByte(int value)
        {
            if (value < 0) return 0;
            if (value > 255) return 255;
            else return (byte)value;
        }
    }

    public interface ISaveImage
    {
        /// <summary>
        /// Bitmap mentese fajlba
        /// </summary>
        /// <param name="fileName">Fajlnev eleresi uttal</param>
        /// <param name="bitmap">Bitmap object</param>
        void SaveImage(string fileName, Bitmap bitmap);
    }

    class TimeMeasure
    {
        private DateTime dT;

        public TimeMeasure()
        {
            dT = DateTime.Now;
        }

        public string getTimeSpan()
        {
            TimeSpan tS = DateTime.Now - dT;
            return tS.TotalMilliseconds + " millisec (" + Math.Floor(tS.TotalMilliseconds / 1000) + " sec)";
        }
    }

}
