/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Filters
{
    public partial class frmFilterConvolution : Form
    {
        private FilterConvolution filter;

        public FilterConvolution OwnFilter
        {
            set { filter = value; }
        }


        private SortedList<string, int[]> kernels = new SortedList<string, int[]>();
        private SortedList<int, TextBox> controls = new SortedList<int, TextBox>();

        public frmFilterConvolution()
        {
            InitializeComponent();
        }

        private void frmFilterConvolution_Load(object sender, EventArgs e)
        {
            controls.Add(0, text11);
            controls.Add(1, text12);
            controls.Add(2, text13);
            controls.Add(3, text21);
            controls.Add(4, text22);
            controls.Add(5, text23);
            controls.Add(6, text31);
            controls.Add(7, text32);
            controls.Add(8, text33);
            controls.Add(9, textScale);
            controls.Add(10, textOffset);
            kernels.Add("Elmoss (Box blur)", new int[11] { 1, 1, 1, 1, 1, 1, 1, 1, 1, 9, 0 });
            kernels.Add("Elmoss (Gaussian blur)", new int[11] { 1, 2, 1, 2, 4, 2, 1, 2, 1, 16, 0 });
            kernels.Add("lests (Sharpen)", new int[11] { 0, -1, 0, -1, 5, -1, 0, -1, 0, 1, 0 });
            kernels.Add("lkeress (Find edges)", new int[11] { 0, -1, 0, -1, 4, -1, 0, -1, 0, 1, 0 });
            kernels.Add("Domborts (Emboss)", new int[11] { -2, -1, 0, -1, 1, 1, 0, 1, 2, 1, 0 });
            kernels.Add("Tbb kontraszt (More contrast)", new int[11] { 0, 0, 0, 0, 5, 0, 0, 0, 0, 1, -640 });
            kernels.Add("Identikus (Identical)", new int[11] { 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0 });
            

            foreach (string str in kernels.Keys)
            {
                comboConvKernels.Items.Add(str);
            }
            comboConvKernels.SelectedIndex = 0;
        }

        private void comboConvKernels_SelectedIndexChanged(object sender, EventArgs e)
        {
            for (int i = 0; i < 11; i++)
            {
                controls[i].Text = kernels[(string)comboConvKernels.SelectedItem][i].ToString();
            }
        }

        private void bCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            try
            {
                filter.RenderImage(getConvMatrix());
                this.DialogResult = DialogResult.OK;
            }
            catch
            {
                MessageBox.Show("Hibs mezkitlts!");
            }
        }

        private void bPreview_Click(object sender, EventArgs e)
        {
            try
            {
                filter.RenderImage(getConvMatrix());
            }
            catch
            {
                MessageBox.Show("Hibs mezkitlts!");
            }
        }

        private FilterConvolution.ConvolutionMatrix getConvMatrix()
        {
            FilterConvolution.ConvolutionMatrix c;
            c.C11 = Convert.ToInt32(text11.Text);
            c.C12 = Convert.ToInt32(text12.Text);
            c.C13 = Convert.ToInt32(text13.Text);
            c.C21 = Convert.ToInt32(text21.Text);
            c.C22 = Convert.ToInt32(text22.Text);
            c.C23 = Convert.ToInt32(text23.Text);
            c.C31 = Convert.ToInt32(text31.Text);
            c.C32 = Convert.ToInt32(text32.Text);
            c.C33 = Convert.ToInt32(text33.Text);
            c.Offset = Convert.ToInt32(textOffset.Text);
            c.Scale = Convert.ToInt32(textScale.Text);
            return c;
        }

    }
}