﻿/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Filters
{
    public partial class frmFilterMagnifier : Form
    {
        private FilterMagnifier filter;
        private int xCoord = 0, yCoord = 0;
        private int mouseXCoord = -999, mouseYCoord = -999;

        public FilterMagnifier OwnFilter
        {
            set { filter = value; }
        }

        public Image PreviewImage
        {
            set { picPreview.Image = value; }
        }

        public frmFilterMagnifier()
        {
            InitializeComponent();
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            filter.RenderImage(xCoord, yCoord, trackRadius.Value, trackScale.Value, checkQuality.Checked);
            this.DialogResult = DialogResult.OK;
        }

        private void bCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        private Rectangle getPreviewRectangle(PictureBox pBox) // with Reflector:)
        {
            Rectangle rectangle = new Rectangle();
            Size size = pBox.Image.Size;
            float num = Math.Min((float)(((float)pBox.ClientRectangle.Width) / ((float)size.Width)), (float)(((float)pBox.ClientRectangle.Height) / ((float)size.Height)));
            rectangle.Width = (int)(size.Width * num);
            rectangle.Height = (int)(size.Height * num);
            rectangle.X = (pBox.ClientRectangle.Width - rectangle.Width) / 2;
            rectangle.Y = (pBox.ClientRectangle.Height - rectangle.Height) / 2;
            return rectangle;
        }

        private void picPreview_MouseClick(object sender, MouseEventArgs e)
        {
            Rectangle r = getPreviewRectangle(picPreview);
            xCoord = (int)((float)picPreview.Image.Width / r.Width * (e.X - r.Left));
            yCoord = (int)((float)picPreview.Image.Height / r.Height * (e.Y - r.Top));
            mouseXCoord = e.X;
            mouseYCoord = e.Y;
            picPreview.Refresh();
            filter.RenderImage(xCoord, yCoord, trackRadius.Value, trackScale.Value, checkQuality.Checked);
        }

        private void bPreview_Click(object sender, EventArgs e)
        {
            filter.RenderImage(xCoord, yCoord, trackRadius.Value, trackScale.Value, checkQuality.Checked);
            picPreview.Refresh();
        }

        private void picPreview_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.FillRectangle(new SolidBrush(SystemColors.ControlDark), new Rectangle(0, 0, picPreview.Width, picPreview.Height));
            e.Graphics.DrawImage(picPreview.Image, getPreviewRectangle(picPreview));
            float r = (float)getPreviewRectangle(picPreview).Width / picPreview.Image.Width * trackRadius.Value;
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            e.Graphics.FillEllipse(new SolidBrush(Color.FromArgb(192, 255, 255, 255)), (float)mouseXCoord - r, (float)mouseYCoord - r, 2 * r, 2 * r);
            e.Graphics.DrawEllipse(new Pen(new SolidBrush(Color.Black), 1), (float)mouseXCoord - r, (float)mouseYCoord - r, 2 * r, 2 * r);
         }

    }
}