/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace Filters
{
    class FilterAverage : IFilter
    {
        public unsafe void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            TimeMeasure tM = new TimeMeasure();
            progressEvent(this, ProgressEventArgs.InProgress);

            BitmapData bD = bitmap.LockBits(new Rectangle(0, 0, bitmap.Width, bitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);

            byte* p = (byte*)bD.Scan0;
            int offset = bD.Stride - bD.Width * 3;
            int r = 0, g = 0, b = 0;

            for (int i = 0; i < bD.Height; ++i)
            {
                for (int j = 0; j < bD.Width; ++j)
                {
                    b += p[0];
                    g += p[1];
                    r += p[2];
                    p += 3;
                }
                p += offset;
            }

            r /= (bitmap.Width * bitmap.Height);
            g /= (bitmap.Width * bitmap.Height);
            b /= (bitmap.Width * bitmap.Height);
            
            bitmap.UnlockBits(bD);

            Graphics graphics = Graphics.FromImage(bitmap);
            graphics.FillRectangle(new SolidBrush(Color.FromArgb(r, g, b)), new Rectangle(0, 0, bitmap.Width, bitmap.Height));
            graphics.Dispose();

            progressEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            finishedEvent(this, new EventArgs());
        }

    }
}
