/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Filters
{
    public class FilterColorBalance : IFilter
    {
        private frmFilterColorBalance form = new frmFilterColorBalance();
        Bitmap srcBitmap, destBitmap;
        ProgressEventHandler pEvent;
        FinishedEventHandler fEvent;


        public unsafe void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            throw new Exception("Nincs teljesen kesz:)");
            srcBitmap = (Bitmap)bitmap.Clone();
            destBitmap = bitmap;
            pEvent = progressEvent;
            fEvent = finishedEvent;

            form.OwnFilter = this;

            DialogResult dR = form.ShowDialog();
            if (dR != DialogResult.OK) Restore();
        }

        public void Restore()
        {
            BitmapOperations.CopyBitmapToBitmap(srcBitmap, destBitmap);
            fEvent(this, new EventArgs());
        }

        public unsafe void RenderImage(int redLevel, int greenLevel, int blueLevel, bool preserveLuminosity)
        {
            TimeMeasure tM = new TimeMeasure();
            pEvent(this, ProgressEventArgs.InProgress);

            BitmapData bDS = srcBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData bDD = destBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            byte* pS = (byte*)bDS.Scan0;
            byte* pD = (byte*)bDD.Scan0;
            int stride = bDD.Stride;
            int offset = stride - bDD.Width * 3;


            for (int i = 0; i < bDD.Height; ++i)
            {
                if (i % (bDD.Height / 10) == 0)
                {
                    pEvent(this, new ProgressEventArgs((float)i / bDD.Height));
                }

                // OPTIMALIZALNI!!!
                for (int j = 0; j < bDD.Width; ++j)
                {
                    int iB = pS[0], iG = pS[1], iR = pS[2];

                    if (!preserveLuminosity)
                    {
                        iB += blueLevel;
                        iG += greenLevel;
                        iR += redLevel;
                    }
                    else
                    {
                        float iB_ = pS[0] / 255.0f, iG_ = pS[1] / 255.0f, iR_ = pS[2] / 255.0f;
                        float iRL = redLevel / 255.0f;
                        float iGL = greenLevel / 255.0f;
                        float iBL = blueLevel / 255.0f;

                        // ITU-R BT.709 szinter
                        // Y = 0.2126 R + 0.7152 G + 0.0722 B
                        //float y = 0.2126f * iR_ + 0.7152f * iG_ + 0.0722f * iB_;
                        int y = 13932 * iR + 46870 * iG + 4733 * iB;
                        
                        //float iB2 = iB_ * (1 + 0.85f * iBL - 0.15f * (iRL + iGL));
                        int iB2 = iB * (255 + 217 * redLevel - 38 * (redLevel  + greenLevel ));
                        //float iG2 = iG_ * (1 + 0.85f * iGL - 0.15f * (iRL + iBL));
                        int iG2 = iG * (255 + 217 * greenLevel - 38 * (redLevel  + blueLevel ));
                        //float iR2 = iR_ * (1 + 0.85f * iRL - 0.15f * (iGL + iBL));
                        int iR2 = iR * (255 + 217 * redLevel- 38 * (greenLevel + blueLevel));

                        //float newY = 0.2126f * iR2 + 0.7152f * iG2 + 0.0722f * iB2;
                        int newY = (int)(54 * iR + 182 * iG + 19 * iB);

                        float ratio;
                        if (newY <= 0) ratio = 0;
                        else ratio = y / newY >> 8;
                        iB = (int)(iB2 * ratio)>>8;
                        iG = (int)(iG2 * ratio)>>8;
                        iR = (int)(iR2 * ratio)>>8;
                    }

                    if (iB < 0) iB = 0; else if (iB > 255) iB = 255;
                    if (iG < 0) iG = 0; else if (iG > 255) iG = 255;
                    if (iR < 0) iR = 0; else if (iR > 255) iR = 255;

                    pD[0] = (byte)iB;
                    pD[1] = (byte)iG;
                    pD[2] = (byte)iR;

                    pS += 3;
                    pD += 3;
                }
                pS += offset;
                pD += offset;
            }

            srcBitmap.UnlockBits(bDS);
            destBitmap.UnlockBits(bDD);

            pEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            fEvent(this, new EventArgs());
        }

    }
}
