/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Filters
{
    public class FilterConvolution : IFilter
    {
        public struct ConvolutionMatrix
        {
            public int C11;
            public int C12;
            public int C13;
            public int C21;
            public int C22;
            public int C23;
            public int C31;
            public int C32;
            public int C33;
            public int Scale;
            public int Offset;
        }

        private frmFilterConvolution form = new frmFilterConvolution();
        Bitmap srcBitmap, destBitmap;

        ProgressEventHandler pEvent;
        FinishedEventHandler fEvent;
        

        public void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            srcBitmap = (Bitmap)bitmap.Clone();
            destBitmap = bitmap;
            pEvent = progressEvent;
            fEvent = finishedEvent;

            form.OwnFilter = this;

            DialogResult dR = form.ShowDialog();
            if (dR != DialogResult.OK) Restore();
        }

        public void Restore()
        {
            BitmapOperations.CopyBitmapToBitmap(srcBitmap, destBitmap);
            fEvent(this, new EventArgs());
        }

        public unsafe void RenderImage(ConvolutionMatrix c)
        {
            TimeMeasure tM = new TimeMeasure();
            pEvent(this, ProgressEventArgs.InProgress);

            BitmapData bDS = srcBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData bDD = destBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            byte* pS = (byte*)bDS.Scan0;
            byte* pD = (byte*)bDD.Scan0;
            int stride = bDD.Stride;
            int offset = stride - bDD.Width * 3;

            pS += stride + 3;
            pD += stride + 3;
            for (int i = 0 + 1; i < bDD.Height - 1; ++i)
            {
                if (i % (bDD.Height / 10) == 0)
                {
                    pEvent(this, new ProgressEventArgs((float)i / bDD.Height));
                }

                for (int j = 0 + 3; j < (bDD.Width - 1) * 3; ++j)
                {

                    int s = c.C11 * pS[-stride - 3] +
                            c.C12 * pS[-stride] +
                            c.C13 * pS[-stride + 3] +
                            c.C21 * pS[-3] +
                            c.C22 * pS[0] +
                            c.C23 * pS[3] +
                            c.C31 * pS[stride - 3] +
                            c.C32 * pS[stride] +
                            c.C33 * pS[stride + 3];
                    s = s / c.Scale + c.Offset;
                    if (s < 0) s = 0;
                    if (s > 255) s = 255;
                    pD[0] = (byte)s;

                    pD++;
                    pS++;
                }
                pD += offset + 6;
                pS += offset + 6;
            }

            srcBitmap.UnlockBits(bDS);
            destBitmap.UnlockBits(bDD);

            pEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            fEvent(this, new EventArgs());
        }

    }
}
