/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace Filters
{
    class FilterInvert : IFilter
    {
        public unsafe void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            TimeMeasure tM = new TimeMeasure();
            progressEvent.Invoke(this, ProgressEventArgs.InProgress);

            BitmapData bD = bitmap.LockBits(new Rectangle(0, 0, bitmap.Width, bitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            unchecked
            {
                byte* p = (byte*)bD.Scan0;
                int offset = bD.Stride - bD.Width * 3;

                for (int i = 0; i < bD.Height; ++i)
                {
                    for (int j = 0; j < bD.Width; ++j)
                    {
                        p[0] = (byte)(255 - p[0]);
                        p[1] = (byte)(255 - p[1]);
                        p[2] = (byte)(255 - p[2]);
                        p += 3;
                    }
                    p += offset;
                }
            }

            bitmap.UnlockBits(bD);

            progressEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            finishedEvent(this, new EventArgs());
        }
    }
}
