/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace Filters
{
    class FilterRotate180 : IFilter
    {
        public unsafe void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            TimeMeasure tM = new TimeMeasure();
            progressEvent(this, ProgressEventArgs.InProgress);

            Bitmap src = (Bitmap)bitmap.Clone();
            BitmapData bDS = src.LockBits(new Rectangle(0, 0, src.Width, src.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData bDD = bitmap.LockBits(new Rectangle(0, 0, src.Width, src.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            byte* pS = (byte*)bDS.Scan0;
            byte* pD = (byte*)bDD.Scan0;
            int stride = bDD.Stride;
            int offset = stride - bDD.Width * 3;

            for (int i = 0; i < bDD.Height; ++i)
            {
                for (int j = 0; j < bDD.Width; ++j)
                {
                    pD[0] = pS[(bDD.Height - 1 - i) * stride + (bDD.Width - 1 - j) * 3];
                    pD[1] = pS[(bDD.Height - 1 - i) * stride + (bDD.Width - 1 - j) * 3 + 1];
                    pD[2] = pS[(bDD.Height - 1 - i) * stride + (bDD.Width - 1 - j) * 3 + 2];
                    pD += 3;
                }
                pD += offset;
            }

            src.UnlockBits(bDS);
            bitmap.UnlockBits(bDD);

            src.Dispose();

            progressEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            finishedEvent(this, new EventArgs());
        }
    }
}
