﻿/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Filters
{
    public class FilterTreshold : IFilter
    {
        private frmFilterTreshold form = new frmFilterTreshold();
        Bitmap srcBitmap, destBitmap;
        ProgressEventHandler pEvent;
        FinishedEventHandler fEvent;


        public unsafe void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            srcBitmap = (Bitmap)bitmap.Clone();
            destBitmap = bitmap;
            pEvent = progressEvent;
            fEvent = finishedEvent;

            form.OwnFilter = this;

            DialogResult dR = form.ShowDialog();
            if (dR != DialogResult.OK) Restore();
        }

        public void Restore()
        {
            BitmapOperations.CopyBitmapToBitmap(srcBitmap, destBitmap);
            fEvent(this, new EventArgs());
        }

        public unsafe void RenderImage(int treshold)
        {
            TimeMeasure tM = new TimeMeasure();
            pEvent(this, ProgressEventArgs.InProgress);

            BitmapData bDS = srcBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData bDD = destBitmap.LockBits(new Rectangle(0, 0, srcBitmap.Width, srcBitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            byte* pS = (byte*)bDS.Scan0;
            byte* pD = (byte*)bDD.Scan0;
            int stride = bDD.Stride;
            int offset = stride - bDD.Width * 3;

            for (int i = 0; i < bDD.Height; ++i)
            {
                for (int j = 0; j < bDD.Width; ++j)
                {
                    int g = ((int)pS[0] + pS[1] + pS[2]) / 3;
                    if (g < treshold)
                    {
                        pD[0] = pD[1] = pD[2] = 0;
                    }
                    else
                    {
                        pD[0] = pD[1] = pD[2] = 255;
                    }

                    pD += 3;
                    pS += 3;
                }
                pD += offset;
                pS += offset;
            }

            srcBitmap.UnlockBits(bDS);
            destBitmap.UnlockBits(bDD);

            pEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            fEvent(this, new EventArgs());
        }
    }
}
