/*
	This file is part of Filters - Grafikus szurok
	Copyright (C) 2008, Istvan Neuwirth <pitta2@gmail.com>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace Filters
{
    class FilterWaves : IFilter
    {
        public unsafe void Render(Bitmap bitmap, ProgressEventHandler progressEvent, FinishedEventHandler finishedEvent)
        {
            TimeMeasure tM = new TimeMeasure();
            progressEvent(this, ProgressEventArgs.InProgress);

            Bitmap src = (Bitmap)bitmap.Clone();
            BitmapData bDS = src.LockBits(new Rectangle(0, 0, src.Width, src.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData bDD = bitmap.LockBits(new Rectangle(0, 0, src.Width, src.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);

            byte* pS = (byte*)bDS.Scan0;
            byte* pD = (byte*)bDD.Scan0;
            int offset = bDD.Stride - bDD.Width * 3;

            for (int i = 0; i < bDD.Height; ++i)
            {
                if (i % (bDD.Height / 10) == 0)
                {
                    progressEvent(this, new ProgressEventArgs((float)i / bDD.Height));
                }

                for (int j = 0; j < bDD.Width; ++j)
                {
                    int newY = i + (int)(Math.Sin((double)j / 25) * 20);

                    if (newY >= 0 && newY < bDD.Height)
                    {
                        pD[0] = pS[newY * bDD.Stride + j * 3];
                        pD[1] = pS[newY * bDD.Stride + j * 3 + 1];
                        pD[2] = pS[newY * bDD.Stride + j * 3 + 2];
                    }
                    else
                    {
                        pD[0] = pD[1] = pD[2] = 0;
                    }
                    pD += 3;
                }
                pD += offset;
            }

            src.UnlockBits(bDS);
            bitmap.UnlockBits(bDD);

            src.Dispose();

            progressEvent(this, new ProgressEventArgs(1, tM.getTimeSpan()));
            finishedEvent(this, new EventArgs());
        }
    }
}
